#!perl

# boxchrs.pl
#
# This program translates IBM PC line-drawing characters in a .txt file into
# and their UTF-8 (Unicode) equivalents.  Input is taken from standard-input,
# output is written to standard-output.
#
# Usage:
#    perl -w boxchrs.pl -b <pcfile.txt >utf8file.txt
#
# or:
#    perl -w boxchrs.pl -m <pcfile.txt >utf8file.txt
#
# You must specify either the -b option or the -m option:
# The -m option says to add the Microsoft-style 3-byte UTF8 prefix to the output.
# The -b option is for 'bare' translation (with no Microsoft-style 3-byte prefix).
#
# This program is tested and known to work with both 16-bit Perl 4.036 for
# MS-DOS (perl4s.exe), ActiveState's Win32 Perl 5.xx, and with 32-bit jperl
# 4.036 for Win32.
#
# If you need a Perl but don't want to go to the trouble of downloading a
# (gigantic) Perl 5.xx installation, you can get a tiny Perl 4.036 for MS-DOS
# here:
#
#    http://www.burtonsys.com/auxil/   (ctrl-F and search for "Perl")
#
# -Dave Burton    <dave@burtonsys.com.nospam>   Tel: +1 919-481-0149
# Uncopyrighted.  2/25/2003, 1/23/2006


# PC line-draw character translations to UTF-8
$boxchr{"\304"} = "\342\224\200";    # $boxchr{''} = '─';
$boxchr{"\263"} = "\342\224\202";    # $boxchr{''} = '│';
$boxchr{"\332"} = "\342\224\214";    # $boxchr{''} = '┌';
$boxchr{"\277"} = "\342\224\220";    # $boxchr{''} = '┐';
$boxchr{"\300"} = "\342\224\224";    # $boxchr{''} = '└';
$boxchr{"\331"} = "\342\224\230";    # $boxchr{''} = '┘';
$boxchr{"\303"} = "\342\224\234";    # $boxchr{''} = '├';
$boxchr{"\264"} = "\342\224\244";    # $boxchr{''} = '┤';
$boxchr{"\302"} = "\342\224\254";    # $boxchr{''} = '┬';
$boxchr{"\301"} = "\342\224\264";    # $boxchr{''} = '┴';
$boxchr{"\305"} = "\342\224\274";    # $boxchr{''} = '┼';
$boxchr{"\315"} = "\342\225\220";    # $boxchr{''} = '═';
$boxchr{"\272"} = "\342\225\221";    # $boxchr{''} = '║';
$boxchr{"\311"} = "\342\225\224";    # $boxchr{''} = '╔';
$boxchr{"\273"} = "\342\225\227";    # $boxchr{''} = '╗';
$boxchr{"\310"} = "\342\225\232";    # $boxchr{''} = '╚';
$boxchr{"\274"} = "\342\225\235";    # $boxchr{''} = '╝';
$boxchr{"\314"} = "\342\225\240";    # $boxchr{''} = '╠';
$boxchr{"\271"} = "\342\225\243";    # $boxchr{''} = '╣';
$boxchr{"\313"} = "\342\225\246";    # $boxchr{''} = '╦';
$boxchr{"\312"} = "\342\225\251";    # $boxchr{''} = '╩';
$boxchr{"\316"} = "\342\225\254";    # $boxchr{''} = '╬';
$boxchr{"\325"} = "\342\225\222";    # $boxchr{''} = '╒';
$boxchr{"\326"} = "\342\225\223";    # $boxchr{''} = '╓';
$boxchr{"\270"} = "\342\225\225";    # $boxchr{''} = '╕';
$boxchr{"\267"} = "\342\225\226";    # $boxchr{''} = '╖';
$boxchr{"\324"} = "\342\225\230";    # $boxchr{''} = '╘';
$boxchr{"\323"} = "\342\225\231";    # $boxchr{''} = '╙';
$boxchr{"\276"} = "\342\225\233";    # $boxchr{''} = '╛';
$boxchr{"\275"} = "\342\225\234";    # $boxchr{''} = '╜';
$boxchr{"\306"} = "\342\225\236";    # $boxchr{''} = '╞';
$boxchr{"\307"} = "\342\225\237";    # $boxchr{''} = '╟';
$boxchr{"\265"} = "\342\225\241";    # $boxchr{''} = '╡';
$boxchr{"\266"} = "\342\225\242";    # $boxchr{''} = '╢';
$boxchr{"\321"} = "\342\225\244";    # $boxchr{''} = '╤';
$boxchr{"\322"} = "\342\225\245";    # $boxchr{''} = '╥';
$boxchr{"\317"} = "\342\225\247";    # $boxchr{''} = '╧';
$boxchr{"\320"} = "\342\225\250";    # $boxchr{''} = '╨';
$boxchr{"\330"} = "\342\225\252";    # $boxchr{''} = '╪';
$boxchr{"\327"} = "\342\225\253";    # $boxchr{''} = '╫';

if (($#ARGV >= 0) && ("-b" eq $ARGV[0])) {
   $bare = 1;
   shift @ARGV;
} elsif (($#ARGV >= 0) && ("-m" eq $ARGV[0])) {
   $bare = 0;
   shift @ARGV;
} else {
   print STDERR "This program translates IBM PC line-drawing characters in a .txt file into\n" .
                "and their UTF-8 (Unicode) equivalents.  Input is taken from standard-input,\n" .
                "output is written to standard-output.\n" .
                "\n" .
                "Usage:\n" .
                "   perl -w boxchrs.pl -b <pcfile.txt >utf8file.txt\n" .
                "or:\n" .
                "   perl -w boxchrs.pl -m <pcfile.txt >utf8file.txt\n" .
                "\n" .
                "You must specify either the -b option or the -m option:\n" .
                "The -m option says to add the Microsoft-style 3-byte UTF8 prefix to the output.\n" .
                "The -b option is for 'bare' translation (with no Microsoft-style 3-byte prefix).\n";
   exit(1);
}


if (!$bare) {
   # Add Microsoft-style UTF-8 prefix to output file:
   # print "﻿";
   print "\357\273\277";
}

while (<>) {
   chop;
   # if ($_ =~ /[ĳڿôͺɻȼָ̹Ӿǵ]/)
   if ($_ =~ /[\304\263\332\277\300\331\303\264\302\301\305\315\272\311\273\310\274\314\271\313\312\316\325\326\270\267\324\323\276\275\306\307\265\266\321\322\317\320\330\327]/) {
      $tmp = '';
      while ($_ =~ /^([^\304\263\332\277\300\331\303\264\302\301\305\315\272\311\273\310\274\314\271\313\312\316\325\326\270\267\324\323\276\275\306\307\265\266\321\322\317\320\330\327]*)([\304\263\332\277\300\331\303\264\302\301\305\315\272\311\273\310\274\314\271\313\312\316\325\326\270\267\324\323\276\275\306\307\265\266\321\322\317\320\330\327])(.*)$/) {
         $tmp .= ($1 . $boxchr{$2});
         $_ = $3;
      }
      $_ = $tmp . $_;
   }
   print $_ . "\n";
}

__END__
